%FreakLab - apply.m
%
%Erstellt von Matthias Hasewend im Rahmen der bung 'Applikationssoftware
%fr Fortgeschrittene' bei Ass. Prof. Winfried Kernbichler, Institut fr
%Theoretische Physik / Computational Physics an der TU Graz im WS 2010/11.
%
%Beschreibung:
%
%Diese Funktion wird augefhrt, wenn der Benutzer im Programmfenster den
%pushbutton1 'Anwenden' drckt. In diesem Fall wird fr das aktuelle
%Bedienfeld entsprechend den darin enthaltenen Spezifikationen die
%Bilddatei eben jener Transformation unterzogen, die bereits im
%Preview-Fenster angezeigt wird. Je nach Gre des Bildes und
%Kompliziertheit der Transformation kann dies einige Sekunden dauern (daher
%gibt es auch das Preview-Fenster).
%
%Nach dem Anwenden der Transformation und der Betrachtung des Resultats im
%Groen Anzeigefenster kann der Benutzer entscheiden, ob er das neue Bild
%bernimmt (confirm.m) oder verwirft (undo.m). Die
%entsprechenden Buttons sind dann aktiv, alle anderen nicht.

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%num gibt an, welches Bedienfeld gerade aktuell ist
function apply(hObject,eventdata,handles,num)

    %Logische Hilfsvariable
    ind = logical(1);

    %Aktion fr Bedienfeld 1
    if(num == 1)
        
        %Wenn Bilddatei zu gro wird...
        if((prod(handles.imsize)*abs(det(handles.afftra))/3) >= handles.limsize)
        
            %...wird eine Warnung ausgegeben
            beep;
            warndlg(sprintf('Dieses Bild wrde bei der Transformation zu gro werden.\nBitte whlen Sie eine andere Transformation.'),'Problem');

            %...und der Indikator auf null gesetzt
            ind = 0;
            
        else
            
            %Sonst berechnen der neuen Bilddatei mit der fr das gesamte 
            %Programm uerst wichtigen Matlabroutine imtransform und der
            %Transformationsmatrix fr diese lineare Abbildung.
            handles.imnew = imtransform(handles.image,maketform('affine',[handles.afftra,zeros(2,1);zeros(1,2),1]));
            

        end
    
    %Aktion fr Bedienfeld 2
    elseif(num == 2)
        
        if((prod(handles.imsize)*handles.xy/(handles.uv*3)) >= handles.limsize)
        
            beep;
            warndlg(sprintf('Dieses Bild wrde bei der Transformation zu gro werden.\nBitte whlen Sie eine andere Transformation.'),'Problem');

            ind = 0;
            
        else

            %Erzeugen der Transformationsdaten fr die holomorphe Abbildung
            %aus der Formel (Polynomsumme)
            tform = tformgen(handles,1);
            
            %Gewinnen des Bild- und Urbild-Zoomfaktors
            uvdat = [-handles.uv,handles.uv];
            xydat = [-handles.xy,handles.xy];

            %Durchfhren der Transformation zu diesen Daten
            handles.imnew = imtransform(handles.image,tform,'UData',uvdat,'VData',uvdat,'XData',xydat,'YData',xydat);

        end
        
    %Aktion fr Bedienfeld 3
    elseif(num == 3)
        
        if((prod(handles.imsize)*handles.xy/(handles.uv*3)) >= handles.limsize)
        
            beep;
            warndlg(sprintf('Dieses Bild wrde bei der Transformation zu gro werden.\nBitte whlen Sie eine andere Transformation.'),'Problem');

            ind = 0;
            
        else

            %Falls die in die Zeile eingegebe Formel fehlerfrei ist...
            try
            
                %...erzeugen der Transformationsdaten fr die komplexe
                %Abbildung
                tform = tformgen(handles,2);
                
                %...gewinnen des Bild- und Urbild-Zoomfaktors
                uvdat = [-handles.uv,handles.uv];
                xydat = [-handles.xy,handles.xy];

                %...durchfhren der Transformation zu diesen Daten
                handles.imnew = imtransform(handles.image,tform,'UData',uvdat,'VData',uvdat,'XData',xydat,'YData',xydat);


            end
        
        end
      
    %Aktion fr Bedienfeld 4
    elseif(num == 4)
        
        %Hier gibt's kein 'zu Gro', da nur die Farbe des Bildes verndert
        %wird
        handles.holtrac(handles.holtrarow) = 1;
        set(handles.popupmenu6,'Value',1);
        
        %Falls die in den Zeilen eingegeben Formeln fehlerfrei sind...
        try
            %...durchfhren der Transformation zu diesen Daten mit dem
            %selbstgebastelten Befehl imtransform_eigen (siehe
            %imtransform_eigen.m)
            handles.imnew = imtransform_eigen(handles.image(:,:,handles.colshift),handles);

        end
        
    end
    
    %Die folgenden Aktionen sind wieder fr alle Bedienfeldnummer gleich
    
    %Falls die Bilddatei bei der Trafo nicht zu gro geworden ist...
    if(ind)
    
        %...plotten in das groe Anzeigefenster
        set(gcf,'CurrentAxes',handles.axes1);
        image(handles.imnew);
        axis off;

        %...Bedienfeld inaktiv schalten
        disenable(hObject,handles,'off',num);

        %...Buttons zum bernehmen oder verwerfen dieses neuen Bildes
        %aktivieren
        set(handles.pushbutton2,'Enable','on');
        set(handles.pushbutton3,'Enable','on');
        set(handles.text1,'BackgroundColor','Red');

        guidata(hObject,handles);
        
    end
    
end